@@ -1,7 +1,7 @@
 import os
+import shutil
 import tempfile
 import typing
-import warnings
 
 import pandas as pd
 
@@ -32,8 +32,7 @@ def available(
         table with name, version and private flag
 
     """
-
-    match = {}
+    databases = []
     for repository in config.REPOSITORIES:
         pattern = f'*/{define.DB}/*/{define.DB}-*.yaml'
         backend = audbackend.create(
@@ -43,38 +42,33 @@ def available(
         )
         for p in backend.glob(pattern):
             name, _, version, _ = p.split('/')[-4:]
-            if name not in match:
-                match[name] = {
-                    'backend': repository.backend,
-                    'host': repository.host,
-                    'repository': repository.name,
-                    'version': [],
-                }
-            match[name]['version'].append(version)
-
-    for name in match:
-        match[name]['version'] = audeer.sort_versions(
-            match[name]['version']
-        )
-        if only_latest:
-            match[name]['version'] = [match[name]['version'][-1]]
-
-    data = []
-    for name in match:
-        for v in match[name]['version']:
-            data.append(
+            databases.append(
                 [
                     name,
-                    match[name]['backend'],
-                    match[name]['host'],
-                    match[name]['repository'],
-                    v,
+                    repository.backend,
+                    repository.host,
+                    repository.name,
+                    version,
                 ]
             )
-    return pd.DataFrame.from_records(
-        data,
+
+    df = pd.DataFrame.from_records(
+        databases,
         columns=['name', 'backend', 'host', 'repository', 'version'],
-    ).set_index('name')
+    )
+    if only_latest:
+        # Pick latest version for every database, see
+        # https://stackoverflow.com/a/53842408
+        df = df[
+            df['version'] == df.groupby('name')['version'].transform(
+                lambda x: audeer.sort_versions(x)[-1]
+            )
+        ]
+    else:
+        # Sort by version
+        df = df.sort_values(by=['version'], key=audeer.sort_versions)
+    df = df.sort_values(by=['name'])
+    return df.set_index('name')
 
 
 def cached(
@@ -113,8 +107,19 @@ def cached(
             flavor_id_paths = audeer.list_dir_names(version_path)
 
             # Skip old audb cache (e.g. 1 as flavor)
-            if audeer.list_file_names(version_path):  # pragma: no cover
-                continue
+            files = audeer.list_file_names(version_path)
+            deps_path = os.path.join(version_path, define.DEPENDENCIES_FILE)
+            deps_path_cached = os.path.join(
+                version_path,
+                define.CACHED_DEPENDENCIES_FILE,
+            )
+            if deps_path not in files and deps_path_cached not in files:
+                # Skip all cache entries
+                # that don't contain a db.csv or db.pkl file
+                # as those stem from audb<1.0.0.
+                # We only look for db.csv
+                # as we switched to db.pkl with audb>=1.0.5
+                continue  # pragma: no cover
 
             for flavor_id_path in flavor_id_paths:
                 flavor_id = os.path.basename(flavor_id_path)
@@ -183,12 +188,15 @@ def dependencies(
         name: str,
         *,
         version: str = None,
+        cache_root: str = None,
 ) -> Dependencies:
     r"""Database dependencies.
 
     Args:
         name: name of database
         version: version string
+        cache_root: cache folder where databases are stored.
+            If not set :meth:`audb.default_cache_root` is used
 
     Returns:
         dependency object
@@ -198,15 +206,36 @@ def dependencies(
         version = latest_version(name)
     backend = lookup_backend(name, version)
 
-    with tempfile.TemporaryDirectory() as root:
-        archive = backend.join(name, define.DB)
-        deps_path = backend.get_archive(
-            archive,
-            root,
-            version,
-        )[0]
-        deps_path = os.path.join(root, deps_path)
-        deps = Dependencies()
+    cache_roots = [
+        default_cache_root(True),  # check shared cache first
+        default_cache_root(False),
+    ] if cache_root is None else [cache_root]
+    for cache_root in cache_roots:
+        deps_root = audeer.safe_path(
+            os.path.join(
+                cache_root,
+                name,
+                version,
+            )
+        )
+        if os.path.exists(deps_root):
+            break
+
+    audeer.mkdir(deps_root)
+    deps_path = os.path.join(deps_root, define.CACHED_DEPENDENCIES_FILE)
+
+    deps = Dependencies()
+    if not os.path.exists(deps_path):
+        with tempfile.TemporaryDirectory() as tmp_root:
+            archive = backend.join(name, define.DB)
+            backend.get_archive(
+                archive,
+                tmp_root,
+                version,
+            )
+            deps.load(os.path.join(tmp_root, define.DEPENDENCIES_FILE))
+            deps.save(deps_path)
+    else:
         deps.load(deps_path)
 
     return deps
@@ -429,7 +458,7 @@ def remove_media(
                             )
 
                     # mark file as removed
-                    deps.remove(file)
+                    deps._remove(file)
                     upload = True
 
             # upload dependencies
