@@ -1,3 +1,4 @@
+import errno
 import os
 import typing
 
@@ -10,27 +11,66 @@ from audb.core import define
 
 
 class Dependencies:
-    r"""Hold dependencies of a database.
+    r"""Dependencies of a database.
+
+    :class:`audb.Dependencies` gathers all files
+    a database contains
+    and metadata about them
+    in a single object.
+    The metadata contains information
+    about the single files
+    like duration,
+    but also what version of the file is required.
+
+    The dependencies of a database can be requested with
+    :func:`audb.dependencies`.
+
+    Example:
+        >>> deps = Dependencies()
+        >>> deps()
+        Empty DataFrame
+        Columns: [archive, bit_depth, channels, checksum, duration, format, removed, sampling_rate, type, version]
+        Index: []
+        >>> # Request dependencies for emodb 1.1.0
+        >>> deps = audb.dependencies('emodb', version='1.1.0')
+        >>> # List all files or archives
+        >>> deps.files[:3]
+        ['db.emotion.csv', 'db.files.csv', 'wav/03a01Fa.wav']
+        >>> deps.archives[:2]
+        ['005d2b91-5317-0c80-d602-6d55f0323f8c', '014f82d8-3491-fd00-7397-c3b2ac3b2875']
+        >>> # Access properties for a given file
+        >>> deps.archive('wav/03a01Fa.wav')
+        'c1f5cc6f-6d00-348a-ba3b-4adaa2436aad'
+        >>> deps.duration('wav/03a01Fa.wav')
+        1.89825
+        >>> deps.removed('wav/03a01Fa.wav')
+        False
+        >>> # Check if a file is part of the dependencies
+        >>> 'wav/03a01Fa.wav' in deps
+        True
+
+    """  # noqa: E501
 
-    """
     def __init__(self):
-        self._data = {}
+        data = {}
+        for name, dtype in zip(
+                define.DEPEND_FIELD_NAMES.values(),
+                define.DEPEND_FIELD_DTYPES.values(),
+        ):
+            data[name] = pd.Series(dtype=dtype)
+        self._df = pd.DataFrame(data)
 
     def __call__(self) -> pd.DataFrame:
-        r"""Create dependency table.
+        r"""Return dependencies as a table.
 
         Returns:
             table with dependencies
 
         """
-        return pd.DataFrame.from_dict(
-            self._data,
-            orient='index',
-            columns=list(define.DEPEND_FIELD_NAMES.values()),
-        )
+        return self._df
 
-    def __contains__(self, file: str):
-        r"""Check if file exists.
+    def __contains__(self, file: str) -> bool:
+        r"""Check if file is part of dependencies.
 
         Args:
             file: relative file path
@@ -39,7 +79,7 @@ class Dependencies:
             ``True`` if a dependency to the file exists
 
         """
-        return file in self._data
+        return file in self._df.index
 
     def __getitem__(self, file: str) -> typing.List:
         r"""File information.
@@ -51,7 +91,13 @@ class Dependencies:
             list with meta information
 
         """
-        return self._data[file]
+        return list(self._df.loc[file])
+
+    def __len__(self) -> int:
+        return len(self._df)
+
+    def __str__(self) -> str:
+        return self._df.to_string()
 
     @property
     def archives(self) -> typing.List[str]:
@@ -62,17 +108,7 @@ class Dependencies:
 
         """
         archives = [self.archive(file) for file in self.files]
-        return list(set(archives))
-
-    @property
-    def data(self) -> typing.Dict[str, typing.List]:
-        r"""Get table data.
-
-        Returns:
-            dictionary with table entries
-
-        """
-        return self._data
+        return sorted(list(set(archives)))
 
     @property
     def files(self) -> typing.List[str]:
@@ -82,7 +118,7 @@ class Dependencies:
             list of files
 
         """
-        return list(self._data)
+        return list(self._df.index)
 
     @property
     def media(self) -> typing.List[str]:
@@ -107,7 +143,7 @@ class Dependencies:
 
         """
         select = [
-            file for file in self.media if self.is_removed(file)
+            file for file in self.media if self.removed(file)
         ]
         return select
 
@@ -139,79 +175,6 @@ class Dependencies:
         ]
         return select
 
-    def add_media(
-            self,
-            root: str,
-            file: str,
-            archive: str,
-            checksum: str,
-            version: str,
-    ):
-        r"""Add media file.
-
-        Args:
-            root: root directory
-            file: relative file path
-            archive: archive name without extension
-            checksum: checksum of file
-            version: version string
-
-        """
-        format = audeer.file_extension(file).lower()
-
-        bit_depth = channels = sampling_rate = 0
-        duration = 0.0
-        if format in define.FORMATS:
-            path = os.path.join(root, file)
-            bit_depth = audiofile.bit_depth(path)
-            channels = audiofile.channels(path)
-            duration = audiofile.duration(path)
-            sampling_rate = audiofile.sampling_rate(path)
-
-        self.data[file] = [
-            archive,
-            bit_depth,
-            channels,
-            checksum,
-            duration,
-            format,
-            0,  # removed
-            sampling_rate,
-            define.DependType.MEDIA,
-            version,
-        ]
-
-    def add_meta(
-            self,
-            file: str,
-            archive: str,
-            checksum: str,
-            version: str,
-    ):
-        r"""Add table file.
-
-        Args:
-            file: relative file path
-            archive: archive name without extension
-            checksum: checksum of file
-            version: version string
-
-        """
-        format = audeer.file_extension(file).lower()
-
-        self.data[file] = [
-            archive,                 # archive
-            0,                       # bit_depth
-            0,                       # channels
-            checksum,                # checksum
-            0.0,                     # duration
-            format,                  # format
-            0,                       # removed
-            0,                       # sampling_rate
-            define.DependType.META,  # type
-            version,                 # version
-        ]
-
     def archive(self, file: str) -> str:
         r"""Name of archive the file belongs to.
 
@@ -284,7 +247,7 @@ class Dependencies:
         """
         return self[file][define.DependField.FORMAT]
 
-    def is_removed(self, file: str) -> bool:
+    def removed(self, file: str) -> bool:
         r"""Check if file is marked as removed.
 
         Args:
@@ -297,17 +260,36 @@ class Dependencies:
         return self[file][define.DependField.REMOVED] != 0
 
     def load(self, path: str):
-        r"""Read dependencies from CSV file.
+        r"""Read dependencies from file.
 
         Clears existing dependencies.
 
         Args:
-            path: path to file
+            path: path to file.
+                File extension can be ``csv`` or ``pkl``.
+
+        Raises:
+            ValueError: if file extension is not ``csv`` or ``pkl``
+            FileNotFoundError: if ``path`` does not exists
 
         """
-        self._data = {}
+        self._df = pd.DataFrame(columns=define.DEPEND_FIELD_NAMES.values())
         path = audeer.safe_path(path)
-        if os.path.exists(path):
+        extension = audeer.file_extension(path)
+        if extension not in ['csv', 'pkl']:
+            raise ValueError(
+                f"File extension of 'path' has to be 'csv' or 'pkl' "
+                f"not '{extension}'"
+            )
+        if not os.path.exists(path):
+            raise FileNotFoundError(
+                errno.ENOENT,
+                os.strerror(errno.ENOENT),
+                path,
+            )
+        if extension == 'pkl':
+            self._df = pd.read_pickle(path)
+        elif extension == 'csv':
             # Data type of dependency columns
             dtype_mapping = {
                 name: dtype for name, dtype in zip(
@@ -318,24 +300,12 @@ class Dependencies:
             # Data type of index
             index = 0
             dtype_mapping[index] = str
-            df = pd.read_csv(
+            self._df = pd.read_csv(
                 path,
                 index_col=index,
                 na_filter=False,
                 dtype=dtype_mapping,
             )
-            self._data = {
-                file: list(row) for file, row in df.iterrows()
-            }
-
-    def remove(self, file: str):
-        r"""Mark file as removed.
-
-        Args:
-            file: relative file path
-
-        """
-        self._data[file][define.DependField.REMOVED] = 1
 
     def sampling_rate(self, file: str) -> int:
         r"""Sampling rate of media file.
@@ -350,14 +320,18 @@ class Dependencies:
         return self[file][define.DependField.SAMPLING_RATE]
 
     def save(self, path: str):
-        r"""Write dependencies to CSV file.
+        r"""Write dependencies to file.
 
         Args:
-            path: path to file
+            path: path to file.
+                File extension can be ``csv`` or ``pkl``.
 
         """
         path = audeer.safe_path(path)
-        self().to_csv(path)
+        if path.endswith('csv'):
+            self._df.to_csv(path)
+        elif path.endswith('pkl'):
+            self._df.to_pickle(path)
 
     def type(self, file: str) -> define.DependType:
         r"""Type of file.
@@ -383,8 +357,108 @@ class Dependencies:
         """
         return self[file][define.DependField.VERSION]
 
-    def __len__(self) -> int:
-        return len(self._data)
+    def _add_media(
+            self,
+            root: str,
+            file: str,
+            version: str,
+            archive: str = None,
+            checksum: str = None,
+    ):
+        r"""Add or update media file.
 
-    def __str__(self) -> str:
-        return self().to_string()
+        If you want to update only the version
+        of an unaltered media file,
+        don't specify ``archive`` and ``checksum``.
+
+        Args:
+            root: root directory
+            file: relative file path
+            archive: archive name without extension
+            checksum: checksum of file
+            version: version string
+
+        """
+        format = audeer.file_extension(file).lower()
+
+        if archive is None:
+            archive = self.archive(file)
+
+        if checksum is None:
+            checksum = self.checksum(file)
+            bit_depth = self.bit_depth(file)
+            channels = self.channels(file)
+            duration = self.duration(file)
+            sampling_rate = self.sampling_rate(file)
+        else:
+            bit_depth = channels = sampling_rate = 0
+            duration = 0.0
+            if format in define.FORMATS:
+                path = os.path.join(root, file)
+                bit_depth = audiofile.bit_depth(path)
+                channels = audiofile.channels(path)
+                duration = audiofile.duration(path)
+                sampling_rate = audiofile.sampling_rate(path)
+
+        self._df.loc[file] = [
+            archive,
+            bit_depth,
+            channels,
+            checksum,
+            duration,
+            format,
+            0,  # removed
+            sampling_rate,
+            define.DependType.MEDIA,
+            version,
+        ]
+
+    def _add_meta(
+            self,
+            file: str,
+            version: str,
+            archive: str,
+            checksum: str,
+    ):
+        r"""Add or update table file.
+
+        Args:
+            file: relative file path
+            archive: archive name without extension
+            checksum: checksum of file
+            version: version string
+
+        """
+        format = audeer.file_extension(file).lower()
+
+        self._df.loc[file] = [
+            archive,                 # archive
+            0,                       # bit_depth
+            0,                       # channels
+            checksum,                # checksum
+            0.0,                     # duration
+            format,                  # format
+            0,                       # removed
+            0,                       # sampling_rate
+            define.DependType.META,  # type
+            version,                 # version
+        ]
+
+    def _drop(self, file: str):
+        r"""Drop file from table.
+
+        Args:
+            file: relative file path
+
+        """
+        self._df.drop(file, inplace=True)
+
+    def _remove(self, file: str):
+        r"""Mark file as removed.
+
+        Args:
+            file: relative file path
+
+        """
+        removed_column = define.DEPEND_FIELD_NAMES[define.DependField.REMOVED]
+        self._df.at[file, removed_column] = 1
