@@ -1,3 +1,4 @@
+from distutils.version import LooseVersion
 import os
 import re
 import shutil
@@ -10,7 +11,9 @@ import audformat
 
 from audb.core import define
 from audb.core.api import (
+    cached,
     default_cache_root,
+    dependencies,
     latest_version,
 )
 from audb.core.dependencies import Dependencies
@@ -21,6 +24,103 @@ from audb.core.utils import (
 )
 
 
+def _cached_versions(
+        name: str,
+        version: str,
+        flavor: Flavor,
+        cache_root: typing.Optional[str],
+) -> typing.Sequence[typing.Tuple[LooseVersion, str, Dependencies]]:
+    r"""Find other cached versions of same flavor."""
+
+    df = cached(cache_root=cache_root)
+    df = df[df.name == name]
+
+    cached_versions = []
+    for flavor_root, row in df.iterrows():
+        if row['flavor_id'] == flavor.short_id:
+            if row['version'] == version:
+                continue
+            deps = dependencies(
+                name,
+                version=row['version'],
+                cache_root=cache_root,
+            )
+            # as it is more likely we find files
+            # in newer versions, push them to front
+            cached_versions.insert(
+                0,
+                (
+                    LooseVersion(row['version']),
+                    flavor_root,
+                    deps,
+                ),
+            )
+
+    return cached_versions
+
+
+def _cached_files(
+        files: typing.Sequence[str],
+        deps: Dependencies,
+        cached_versions: typing.Sequence[
+            typing.Tuple[LooseVersion, str, Dependencies],
+        ],
+        flavor: Flavor,
+        verbose: bool,
+) -> (typing.Sequence[typing.Union[str, str]], typing.Sequence[str]):
+    r"""Find cached files."""
+
+    cached_files = []
+    missing_files = []
+
+    for file in audeer.progress_bar(
+            files,
+            desc='Cached files',
+            disable=not verbose,
+    ):
+        found = False
+        file_version = LooseVersion(deps.version(file))
+        for cache_version, cache_root, cache_deps in cached_versions:
+            if cache_version >= file_version:
+                if deps.checksum(file) == cache_deps.checksum(file):
+                    path = os.path.join(cache_root, file)
+                    if flavor.format is not None:
+                        path = audeer.replace_file_extension(
+                            path,
+                            flavor.format,
+                        )
+                    if os.path.exists(path):
+                        found = True
+                        break
+        if found:
+            if flavor.format is not None:
+                file = audeer.replace_file_extension(
+                    file,
+                    flavor.format,
+                )
+            cached_files.append((cache_root, file))
+        else:
+            missing_files.append(file)
+
+    return cached_files, missing_files
+
+
+def _copy_file(
+        file: str,
+        root_src: str,
+        root_tmp: str,
+        root_dst: str,
+):
+    r"""Copy file."""
+    src_path = os.path.join(root_src, file)
+    tmp_path = os.path.join(root_tmp, file)
+    dst_path = os.path.join(root_dst, file)
+    audeer.mkdir(os.path.dirname(tmp_path))
+    audeer.mkdir(os.path.dirname(dst_path))
+    shutil.copy(src_path, tmp_path)
+    _move_file(root_tmp, root_dst, file)
+
+
 def _database_check_complete(
         db: audformat.Database,
         db_root: str,
@@ -34,7 +134,7 @@ def _database_check_complete(
             if not os.path.exists(os.path.join(db_root, table)):
                 return False
         for media in deps.media:
-            if not deps.is_removed(media):
+            if not deps.removed(media):
                 path = os.path.join(db_root, media)
                 path = flavor.destination(path)
                 if not os.path.exists(path):
@@ -111,23 +211,6 @@ def _database_root(
     return db_root, db_root_tmp
 
 
-def _dependencies(
-        db_root: str,
-        db_root_tmp: str,
-        name: str,
-        version: str,
-        backend: audbackend.Backend,
-) -> Dependencies:
-    deps_path = os.path.join(db_root, define.DEPENDENCIES_FILE)
-    if not os.path.exists(deps_path):
-        archive = backend.join(name, define.DB)
-        backend.get_archive(archive, db_root_tmp, version)
-        _move_file(db_root_tmp, db_root, define.DEPENDENCIES_FILE)
-    deps = Dependencies()
-    deps.load(deps_path)
-    return deps
-
-
 def _fix_media_ext(
         tables: typing.Sequence[audformat.Table],
         format: str,
@@ -173,7 +256,7 @@ def _full_path(
             )
 
 
-def _get_media(
+def _get_media_from_backend(
         db: audformat.Database,
         media: typing.Sequence[str],
         db_root: str,
@@ -184,9 +267,7 @@ def _get_media(
         num_workers: typing.Optional[int],
         verbose: bool,
 ):
-    r"""Get media."""
-    if not media:
-        return
+    r"""Load media from backend."""
 
     # figure out archives
     archives = set()
@@ -247,11 +328,47 @@ def _get_media(
         params=[([archive, version], {}) for archive, version in archives],
         num_workers=num_workers,
         progress_bar=verbose,
-        task_description='Get media',
+        task_description='Load media',
     )
 
 
-def _get_tables(
+def _get_media_from_cache(
+        media: typing.Sequence[str],
+        db_root: str,
+        db_root_tmp: str,
+        deps: Dependencies,
+        cached_versions: typing.Sequence[
+            typing.Tuple[LooseVersion, str, Dependencies]
+        ],
+        flavor: Flavor,
+        num_workers: int,
+        verbose: bool,
+) -> typing.Sequence[str]:
+    r"""Copy media from cache."""
+
+    cached_media, missing_media = _cached_files(
+        media,
+        deps,
+        cached_versions,
+        flavor,
+        verbose,
+    )
+
+    def job(cache_root: str, file: str):
+        _copy_file(file, cache_root, db_root_tmp, db_root)
+
+    audeer.run_tasks(
+        job,
+        params=[([root, file], {}) for root, file in cached_media],
+        num_workers=num_workers,
+        progress_bar=verbose,
+        task_description='Copy media',
+    )
+
+    return missing_media
+
+
+def _get_tables_from_backend(
         db: audformat.Database,
         tables: typing.Sequence[str],
         db_root: str,
@@ -261,9 +378,7 @@ def _get_tables(
         num_workers: typing.Optional[int],
         verbose: bool,
 ):
-    r"""Get tables."""
-    if not tables:
-        return
+    r"""Load tables from backend."""
 
     def job(table: str):
         archive = backend.join(
@@ -294,10 +409,51 @@ def _get_tables(
         params=[([table], {}) for table in tables],
         num_workers=num_workers,
         progress_bar=verbose,
-        task_description='Get tables',
+        task_description='Load tables',
     )
 
 
+def _get_tables_from_cache(
+        tables: typing.Sequence[str],
+        db_root: str,
+        db_root_tmp: str,
+        deps: Dependencies,
+        cached_versions: typing.Sequence[
+            typing.Tuple[LooseVersion, str, Dependencies]
+        ],
+        flavor: Flavor,
+        num_workers: int,
+        verbose: bool,
+) -> typing.Sequence[str]:
+    r"""Copy tables from cache."""
+
+    cached_tables, missing_tables = _cached_files(
+        tables,
+        deps,
+        cached_versions,
+        flavor,
+        verbose,
+    )
+
+    def job(cache_root: str, file: str):
+        file_pkl = audeer.replace_file_extension(
+            file,
+            audformat.define.TableStorageFormat.PICKLE,
+        )
+        _copy_file(file, cache_root, db_root_tmp, db_root)
+        _copy_file(file_pkl, cache_root, db_root_tmp, db_root)
+
+    audeer.run_tasks(
+        job,
+        params=[([root, file], {}) for root, file in cached_tables],
+        num_workers=num_workers,
+        progress_bar=verbose,
+        task_description='Copy tables',
+    )
+
+    return missing_tables
+
+
 def _include_exclude_mapping(
         deps: Dependencies,
         include: typing.Optional[typing.Union[str, typing.Sequence[str]]],
@@ -343,12 +499,37 @@ def _media(
     return media
 
 
+def _missing_media(
+        db_root: str,
+        media: typing.Sequence[str],
+        flavor: Flavor,
+        verbose: bool,
+) -> typing.Sequence[str]:
+    missing_media = []
+    for file in audeer.progress_bar(
+            media,
+            desc='Missing media',
+            disable=not verbose
+    ):
+        path = os.path.join(db_root, file)
+        if flavor.format is not None:
+            path = audeer.replace_file_extension(path, flavor.format)
+        if not os.path.exists(path):
+            missing_media.append(file)
+    return missing_media
+
+
 def _missing_tables(
         db_root: str,
         tables: typing.Sequence[str],
+        verbose: bool,
 ) -> typing.Sequence[str]:
     missing_tables = []
-    for table in tables:
+    for table in audeer.progress_bar(
+            tables,
+            desc='Missing tables',
+            disable=not verbose,
+    ):
         file = f'db.{table}.csv'
         path = os.path.join(db_root, file)
         if not os.path.exists(path):
@@ -356,18 +537,6 @@ def _missing_tables(
     return missing_tables
 
 
-def _missing_media(
-        db_root: str,
-        media: typing.Sequence[str],
-) -> typing.Sequence[str]:
-    missing_media = []
-    for file in media:
-        path = os.path.join(db_root, file)
-        if not os.path.exists(path):
-            missing_media.append(file)
-    return missing_media
-
-
 def _move_file(
         root_src: str,
         root_dst: str,
@@ -383,18 +552,10 @@ def _move_file(
 def _remove_media(
         db: audformat.Database,
         deps: Dependencies,
-        flavor: Flavor,
         num_workers: int,
         verbose: bool,
 ):
-    removed_files = []
-    for file in deps.removed_media:
-        if flavor.format is not None:
-            # Rename removed media file to requested format
-            name, _ = os.path.splitext(file)
-            file = f'{name}.{flavor.format}'
-        removed_files.append(file)
-
+    removed_files = deps.removed_media
     if removed_files:
         db.drop_files(
             removed_files,
@@ -503,6 +664,8 @@ def load(
     if version is None:
         version = latest_version(name)
     backend = lookup_backend(name, version)
+    deps = dependencies(name, version=version, cache_root=cache_root)
+    cached_versions = None
 
     flavor = Flavor(
         channels=channels,
@@ -511,7 +674,6 @@ def load(
         bit_depth=bit_depth,
         sampling_rate=sampling_rate,
     )
-
     db_root, db_root_tmp = _database_root(name, version, flavor, cache_root)
 
     if verbose:  # pragma: no cover
@@ -520,7 +682,6 @@ def load(
 
     db = _database_header(db_root, db_root_tmp, name, version, flavor, backend)
     db_is_complete = _database_is_complete(db)
-    deps = _dependencies(db_root, db_root_tmp, name, version, backend)
 
     if 'include' in kwargs or 'exclude' in kwargs:  # pragma: no cover
         include = None
@@ -528,7 +689,7 @@ def load(
             include = kwargs['include']
             warnings.warn(
                 "Argument 'include' is deprecated "
-                "and will be removed with version '1.1.0'. "
+                "and will be removed with version '1.2.0'. "
                 "Use 'media' instead.",
                 category=UserWarning,
                 stacklevel=2,
@@ -538,7 +699,7 @@ def load(
             exclude = kwargs['exclude']
             warnings.warn(
                 "Argument 'exclude' is deprecated "
-                "and will be removed with version '1.1.0'. "
+                "and will be removed with version '1.2.0'. "
                 "Use 'media' instead.",
                 category=UserWarning,
                 stacklevel=2,
@@ -551,9 +712,40 @@ def load(
 
     # load missing tables
     if not db_is_complete:
-        missing_tables = _missing_tables(db_root, requested_tables)
-        _get_tables(db, missing_tables, db_root, db_root_tmp, deps,
-                    backend, num_workers, verbose)
+        missing_tables = _missing_tables(
+            db_root,
+            requested_tables,
+            verbose,
+        )
+        if missing_tables:
+            if cached_versions is None:
+                cached_versions = _cached_versions(
+                    name,
+                    version,
+                    flavor,
+                    cache_root,
+                )
+            if cached_versions:
+                missing_tables = _get_tables_from_cache(
+                    missing_tables,
+                    db_root,
+                    db_root_tmp,
+                    deps,
+                    cached_versions,
+                    flavor,
+                    num_workers,
+                    verbose,
+                )
+            _get_tables_from_backend(
+                db,
+                missing_tables,
+                db_root,
+                db_root_tmp,
+                deps,
+                backend,
+                num_workers,
+                verbose,
+            )
 
     # filter tables
     if tables is not None:
@@ -568,20 +760,53 @@ def load(
 
     # load missing media
     if not db_is_complete and not only_metadata:
-        missing_media = _missing_media(db_root, requested_media)
-        _get_media(db, missing_media, db_root, db_root_tmp, flavor, deps,
-                   backend, num_workers, verbose)
-
-    # fix media extension in tables
-    if flavor.format is not None:
-        _fix_media_ext(db.tables.values(), flavor.format, num_workers, verbose)
+        missing_media = _missing_media(
+            db_root,
+            requested_media,
+            flavor,
+            verbose,
+        )
+        if missing_media:
+            if cached_versions is None:
+                cached_versions = _cached_versions(
+                    name,
+                    version,
+                    flavor,
+                    cache_root,
+                )
+            if cached_versions:
+                missing_media = _get_media_from_cache(
+                    missing_media,
+                    db_root,
+                    db_root_tmp,
+                    deps,
+                    cached_versions,
+                    flavor,
+                    num_workers,
+                    verbose,
+                )
+            _get_media_from_backend(
+                db,
+                missing_media,
+                db_root,
+                db_root_tmp,
+                flavor,
+                deps,
+                backend,
+                num_workers,
+                verbose,
+            )
 
     # filter media
     if media is not None or tables is not None:
         db.pick_files(requested_media)
 
     if not removed_media:
-        _remove_media(db, deps, flavor, num_workers, verbose)
+        _remove_media(db, deps, num_workers, verbose)
+
+    # fix media extension in tables
+    if flavor.format is not None:
+        _fix_media_ext(db.tables.values(), flavor.format, num_workers, verbose)
 
     # convert to full path
     if full_path:
